<?php

/**
 * Payment method callback: settings form.
 */
function commerce_paypal_ec_settings_form($settings = NULL) {
  $form = array();
  // Merge default settings into the stored settings array.
  $default_currency = variable_get('commerce_default_currency', 'USD');
  // Options to autofill the PayPal address parameter
  $address_options = drupal_map_assoc(array('None', 'Billing', 'Shipping'));

  $settings = (array) $settings + array(
    'autofill_address' => 'Billing',
    'business' => '',
    'cart_items_list' => 0,
    'confirm_shipping' => 0,
    'currency_code' => in_array($default_currency, array_keys(commerce_paypal_ec_currencies())) ? $default_currency : 'USD',
    'ipn_logging' => 'notification',
    'landing_page' => 'Billing',
    'language' => 'US',
    'page_style' => 'paypal',
    'payment_action' => 'sale',
    'server' => 'sandbox',
    'solution_type' => 'Sole',
  );

  $form['api_username'] = array(
    '#type' => 'textfield',
    '#title' => t('PayPal API Username'),
    '#description' => t('The PayPal API Username.'),
    '#default_value' => $settings['api_username'],
    '#required' => TRUE,
  );
  $form['api_password'] = array(
    '#type' => 'textfield',
    '#title' => t('PayPal API Password'),
    '#description' => t('The PayPal API Password.'),
    '#default_value' => $settings['api_password'],
    '#required' => TRUE,
  );
  $form['api_signature'] = array(
    '#type' => 'textfield',
    '#title' => t('PayPal API Signature'),
    '#description' => t('The PayPal API Signature.'),
    '#default_value' => $settings['api_signature'],
    '#required' => TRUE,
  );
  $form['currency_code'] = array(
    '#type' => 'select',
    '#title' => t('Currency code'),
    '#description' => t('Transactions can only be processed in one of the listed currencies.'),
    '#options' => commerce_paypal_ec_currencies(),
    '#default_value' => $settings['currency_code'],
  );
  $form['language'] = array(
    '#type' => 'select',
    '#title' => t('PayPal login page language'),
    '#options' => commerce_paypal_ec_countries_locale(),
    '#default_value' => $settings['language'],
  );
  $form['payment_action'] = array(
    '#type' => 'radios',
    '#title' => t('Payment action'),
    '#options' => array(
      'sale' => t('Sale - authorize and capture the funds at the time the payment is processed'),
      'authorization' => t('Authorization - reserve funds on the card to be captured later through your PayPal account'),
    ),
    '#default_value' => $settings['payment_action'],
  );
  $form['solution_type'] = array(
    '#type' => 'radios',
    '#title' => t('Solution type'),
    '#options' => array(
      'Sole' => ('Sole - Buyer does not need to create a PayPal'),
      'Mark' => ('Mark - Buyer must have a PayPal account to check out)'),
    ),
    '#default_value' => $settings['solution_type'],
  );
  $form['landing_page'] = array(
    '#type' => 'radios',
    '#title' => t('Landing page'),
    '#options' => array(
      'Billing' => ('Put focus in the no PayPal account form (credit card) in the PayPal payment page.'),
      'Login' => ('Put focus in the PayPal Login form in the PayPal payment page.'),
    ),
    '#default_value' => $settings['landing_page'],
  );
  $form['autofill_address'] = array(
    '#type' => 'select',
    '#options' => $address_options,
    '#title' => t('Autofill the PayPal Payment form address with the order address. Note that if you use Shipping address you must have activated the commerce_shipping to work correctly. Billing address is by default captured so no additionally steps required.'),
    '#default_value' => $settings['autofill_address'],
  );
  $form['cart_items_list'] = array(
    '#type' => 'checkbox',
    '#title' => t('Send the shopping cart item list to PayPal to display in the summary of PayPal payment page.'),
    '#default_value' => $settings['cart_items_list'],
  );
  $form['confirm_shipping'] = array(
    '#type' => 'checkbox',
    '#title' => t('Confirmed shipping address is mandatory for the PayPal payment.'),
    '#default_value' => $settings['confirm_shipping'],
  );
  $form['page_style'] = array(
    '#type' => 'textfield',
    '#title' => t('Name of your custom page style.'),
    '#description' => t('In you PayPal account you can create a custom page style to add a custom logo, header and background colors to the PayPal payment page, here you can define the name of your page style to give a customized experience to your clients.'),
    '#default_value' => $settings['page_style'],
    '#required' => TRUE,
  );
  $form['server'] = array(
    '#type' => 'radios',
    '#title' => t('PayPal server'),
    '#options' => array(
      'sandbox' => ('Sandbox - use for testing, requires a PayPal Sandbox account'),
      'live' => ('Live - use for processing real transactions'),
    ),
    '#default_value' => $settings['server'],
  );
  $form['ipn_logging'] = array(
    '#type' => 'radios',
    '#title' => t('IPN logging'),
    '#options' => array(
      'notification' => t('Log notifications during IPN validation and processing.'),
      'full_ipn' => t('Log notifications with the full IPN during validation and processing (used for debugging).'),
    ),
    '#default_value' => $settings['ipn_logging'],
  );

  return $form;
}

/**
 * Payment method callback: adds a description message to the submission form.
 */
function commerce_paypal_ec_submit_form($payment_method, $pane_values, $checkout_pane, $order) {
  $icons = commerce_paypal_icons();
  $info = t('Pay quickly and securely with your PayPal account or your debit or credit card.');
  $info .= '<div class="commerce-paypal-icons"><span class="label">' . t('Includes:') . '</span>' . implode(' ', $icons) . '</div>';

  $form['paypal_ec_information'] = array(
    '#markup' => '<div class="commerce-paypal-ec-info">' . $info . '</div>',
  );

  return $form;
}

/**
 * Payment method callback: redirect form, a wrapper around the module's general
 *   use function for building a EC form.
 */
function commerce_paypal_ec_redirect_form($form, &$form_state, $order, $payment_method) {
  // Return an error if the enabling action's settings haven't been configured.
  $api_params = array('api_username', 'api_password', 'api_signature');
  foreach ($api_params as $param_name) {
    if (empty($payment_method['settings'][$param_name])) {
      drupal_set_message(t('PayPal EC is not configured for use. No PayPal !param_name address has been specified.', array('!param_name' => $param_name)), 'error');
      return array();
    }
  }

  $settings = array(
    // Return to the previous page when payment is canceled
    'cancel_return' => url('checkout/' . $order->order_id . '/payment/back/' . $order->data['payment_redirect_key'], array('absolute' => TRUE)),
    // Return to the payment redirect page for processing successful payments
    'return' => url('checkout/' . $order->order_id . '/payment/return/' . $order->data['payment_redirect_key'], array('absolute' => TRUE)),
    'notify_url' => commerce_paypal_ipn_url($payment_method['instance_id']),
  );

  return commerce_paypal_ec_order_form($form, $form_state, $order, $payment_method['settings'] + $settings);
}

/**
 * Builds a PayPal Express Checkout form from an order object.
 *
 * @param $order
 *   The fully loaded order being paid for.
 * @param $settings
 *   An array of settings used to build out the form, including:
 *   - server: which server to use, either sandbox or live
 *   - business: the PayPal e-mail address the payment submits to
 *   - cancel_return: the URL PayPal should send the user to on cancellation
 *   - return: the URL PayPal should send the user to on successful payment
 *   - currency_code: the PayPal currency code to use for this payment if the
 *     total for the order is in a non-PayPal supported currency
 *   - language: the PayPal language code to use on the payment form
 *   - payment_action: the PayPal payment action to use: sale, authorization,
 *     or order
 *   - payment_method: optionally the name of the Commerce payment method to
 *     include in the IPN notify_url
 *
 * @return
 *   A renderable form array.
 */
function commerce_paypal_ec_order_form($form, &$form_state, $order, $settings) {
  global $language;
  $settings += array('payment_method' => '');
  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);

  // look which lang we should use in the PayPal page
  if ($settings['language'] == 'multilingual') {
    // in case of a multilingual site use the current used lang
    $current_lang = $language->language;
    $paypal_locale = commerce_paypal_ec_locale();
    // use the correct localization country code and if not found use US english
    $locale_code = isset($paypal_locale[$current_lang]) ? $paypal_locale[$current_lang] : 'en_US';
  }
  else {
    // no multilingual so use the configured lang
    $locale_code = $settings['language'];
  }

  $request_params = array(
    // Determines whether or not the PayPal pages should display the shipping address
    'ADDROVERRIDE' => 1,
    // Indicates you require the buyer’s shipping PayPal address be a confirmed address
    'REQCONFIRMSHIPPING' => $settings['confirm_shipping'],
    // Determines where or not PayPal displays shipping address fields on the PayPal pages
    'NOSHIPPING' => 1,
    // Your own invoice or tracking number
    'PAYMENTREQUEST_0_INVNUM' => commerce_paypal_ipn_invoice($order),
    // URL to which the buyer’s browser is returned after pay
    'RETURNURL' => $settings['return'],
    // URL to which the buyer is returned if the buyer does not approve the payment
    'CANCELURL' => $settings['cancel_return'],
    'PAYMENTREQUEST_0_NOTIFYURL' => $settings['notify_url'],
    // General description of the purchase
    'PAYMENTREQUEST_0_DESC' => t('Order @order_number at @store', array('@order_number' => $order->order_number, '@store' => variable_get('site_name', url('<front>', array('absolute' => TRUE))))),
    // How you want to obtain payment. (Authorization, Order, Sale)
    'PAYMENTREQUEST_0_PAYMENTACTION' => $settings['payment_action'],
    // Enables the buyer to enter a note to the merchant on the PayPal page during checkout.
    //'ALLOWNOTE' => 1,
    // Locale of pages displayed by PayPal during Express Checkout
    'LOCALECODE' => $locale_code,
    // Name of the Custom Payment Page Style for payment pages associated
    'PAGESTYLE' => $settings['page_style'],
    // Type of checkout flow. (Sole = Buyer does not need to create a PayPal
    // account ~ Mark = Buyer must have a PayPal account to check out)
    'SOLUTIONTYPE' => $settings['solution_type'],
    // Type of PayPal page to display. (Billing = Non-PayPal account ~ Login = PayPal account login)
    'LANDINGPAGE' => $settings['landing_page'],
    // A label that overrides the business name in the PayPal account
    'BRANDNAME' => variable_get('site_name', 'Undefined'),
    // Enables display of “Estimated Total” instead of “Total”
    'TOTALTYPE' => 'Estimated Total',
  );

  // process the address autofill setting
  $address_autofill_params = _commerce_paypal_ec_process_address_autofill($settings, $order_wrapper);
  // merge params to main params array
  $request_params = $request_params + $address_autofill_params;

  // if set, we add all the cart items to display in the PayPal Payement Page
  if ($settings['cart_items_list']) {
    _commerce_paypal_ec_add_items_list_params($order_wrapper, $request_params);
  }

  // add the amount params
  _commerce_paypal_ec_add_amount_params($order, $settings, $request_params);

  $response_data = commerce_paypal_ec_api_call("SetExpressCheckout", $request_params, $settings, $order);

  // handle error response
  if (isset($response_data['ACK']) && $response_data['ACK'] != 'Success') {
    drupal_set_message(t('A problem was found connecting to PayPal. Please try again later.'), 'error');
    watchdog('commerce_paypal_ec', 'The PayPal EC API call failed. !response_data', array('!response_data' => print_r($response_data, TRUE)), WATCHDOG_ERROR);
    return array();
  }

  // handle no token response
  if (empty($response_data['TOKEN'])) {
    drupal_set_message(t('A problem was found getting a Token session at PayPal. Please try again later.'), 'error');
    watchdog('commerce_paypal_ec', 'The PayPal EC API call not provided a Token. !response_data', array('!response_data' => print_r($response_data, TRUE)), WATCHDOG_ERROR);
    return array();
  }

  if (!empty($response_data['TOKEN'])) {
    $form['#action'] = commerce_paypal_ec_server_url($settings['server']) . $response_data['TOKEN'];
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Proceed to PayPal'),
  );

  return $form;
}

/**
 * Payment method callback: redirect form submit
 */
function commerce_paypal_ec_redirect_form_submit($order, $payment_method) {
  if (!$_GET['token'] || !$_GET['PayerID']) {
    drupal_set_message(t('No token or PayerID argument was received from PayPal return URL.', 'error'));
    return;
  }

  $token = $_GET['token'];
  $payer_id = $_GET['PayerID'];
  $request_params = array(
    'TOKEN' => $token,
  );

  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
  // call GetExpressCheckoutDetails to review and execute the payment
  $settings = $payment_method['settings'];
  $response = commerce_paypal_ec_api_call("GetExpressCheckoutDetails", $request_params, $settings, $order);

  // Exit when we don't get a sucess request
  if (isset($response['ACK']) && $response['ACK'] != 'Success') {
    drupal_set_message(t('The Payment at PayPal could not be successful. Try again later.'), 'error');
    commerce_payment_redirect_pane_previous_page($order);
    watchdog('commerce_paypal_ec', 'The PayPal EC API call failed in GetExpressCheckoutDetails. !response_data', array('!response_data' => print_r($response, TRUE)), WATCHDOG_ERROR);
    return;
  }

  // Do the payment
  $request_params = array(
    'TOKEN' => $response['TOKEN'],
    'PAYERID' => $response['PAYERID'],
    'PAYMENTREQUEST_0_AMT' => $response['PAYMENTREQUEST_0_AMT'],
    'PAYMENTREQUEST_0_PAYMENTACTION' => $settings['payment_action'],
    'PAYMENTREQUEST_0_CURRENCYCODE' => $response['PAYMENTREQUEST_0_CURRENCYCODE'],
  );

  // if set, we add all the cart items to display in the PayPal Payement Page
  if ($settings['cart_items_list']) {
    _commerce_paypal_ec_add_items_list_params($order_wrapper, $request_params);
  }

  // Make the call to PayPal to finalize payment
  $do_response = commerce_paypal_ec_api_call("DoExpressCheckoutPayment", $request_params, $settings, $order);

  // Exit when we don't get a sucess request
  if (isset($do_response['ACK']) && $do_response['ACK'] != 'Success') {
    drupal_set_message(t('The Payment at PayPal could not be successful. Try again later.'), 'error');
    commerce_payment_redirect_pane_previous_page($order);
    $debug_data = array('!request_data' => print_r($request_params, TRUE), '!response_data' => print_r($do_response, TRUE));
    watchdog('commerce_paypal_ec', 'The PayPal EC API call failed in DoExpressCheckoutPayment. !request_data \n\n !response_data', $debug_data, WATCHDOG_ERROR);
    return;
  }

  // add order_id to the response array
  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
  $do_response['order_id'] = $order_wrapper->order_id->value();

  // convert response to IPN format and store in paypal_ipn
  $ipn = commerce_paypal_ec_response_to_ipn($do_response);

  if (!empty($ipn)) {
    // Process the response. Instead of use the IPN request we use the DoExpressCheckoutPayment
    // response and we can have the Payment info instantly
    commerce_paypal_ec_paypal_ipn_process($order, $payment_method, $ipn);
    // save the IPN after commerce_paypal_ec_paypal_ipn_process to have the transaction_id
    commerce_paypal_ipn_save($ipn);
  }
}
